#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # PIG et JSON et streaming avec les données vélib - correction avec cloudera
# 
# Correction.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Récupération des données
# 
# Le code suivant télécharge les données nécessaires [data_velib_paris_2014-11-11_22-23.zip](http://www.xavierdupre.fr/enseignement/complements/data_velib_paris_2014-11-11_22-23.zip).

# In[2]:


import pyensae.datasource
import os, datetime
if not os.path.exists("velib"):
    os.mkdir("velib")
files=pyensae.datasource.download_data("data_velib_paris_2014-11-11_22-23.zip", website="xdtd", whereTo="velib")
files[:2]


# ## Connexion au cluster
# 

# In[3]:


import pyensae
get_ipython().run_line_magic('load_ext', 'pyensae')
get_ipython().run_line_magic('load_ext', 'pyenbc')
from pyquickhelper.ipythonhelper import open_html_form
params={"server":"df...fr", "username":"", "password":""}
open_html_form(params=params,title="server + credentials", key_save="params")


# In[4]:


import pyensae
get_ipython().run_line_magic('load_ext', 'pyensae')
password = params["password"]
server = params["server"]
username = params["username"]
client = get_ipython().run_line_magic('remote_open', '')
client


# ## Exercice 1 : convertir les valeurs numériques
# 
# Le programme suivant prend comme argument les colonnes à extraire des fichiers textes qui sont enregistrés au format "python".

# In[5]:


get_ipython().run_cell_magic('PYTHON', 'stream_json.py', '\nimport sys, datetime\ncols = [ _ for _ in sys.argv if ".py" not in _ ]\nfor row in sys.stdin:\n    row = row.strip()\n    if len(row) == 0 :\n        continue\n    js = eval(row)\n    for station in js:\n        vals = [ str(station[c]) for c in cols ]\n        sys.stdout.write(",".join(vals))\n        sys.stdout.write("\\n")\n        sys.stdout.flush()\n')


# On vérifie qu'il fonctionne en lui faisant ingérer les deux lignes suivantes :

# In[6]:


get_ipython().run_cell_magic('runpy', 'stream_json.py name status', "[{'address': 'RUE DES CHAMPEAUX (PRES DE LA GARE ROUTIERE) - 93170 BAGNOLET', 'collect_date': datetime.datetime(2014, 11, 11, 22, 2, 18, 47270), 'lng': 2.416170724425901, 'contract_name': 'Paris', 'name': '31705 - CHAMPEAUX (BAGNOLET)', 'banking': 0, 'lat': 48.8645278209514, 'bonus': 0, 'status': 'OPEN', 'available_bikes': 1, 'last_update': datetime.datetime(2014, 11, 11, 21, 55, 22), 'number': 31705, 'available_bike_stands': 49, 'bike_stands': 50}]\n[{'address': 'RUE DES CHAMPEAUX (PRES DE LA GARE ROUTIERE) - 93170 BAGNOLET', 'collect_date': datetime.datetime(2014, 11, 11, 22, 2, 18, 47270), 'lng': 2.416170724425901, 'contract_name': 'Paris', 'name': '31705 - CHAMPEAUX (BAGNOLET)', 'banking': 0, 'lat': 48.8645278209514, 'bonus': 0, 'status': 'OPEN', 'available_bikes': 1, 'last_update': datetime.datetime(2014, 11, 11, 21, 55, 22), 'number': 31705, 'available_bike_stands': 49, 'bike_stands': 50}]\n")


# On écrit le script PIG qui utilise plus de colonnes :

# In[7]:


get_ipython().run_cell_magic('PIG', 'json_velib.pig', "\nDEFINE pystream `python stream_json.py available_bike_stands available_bikes lat lng name status` \n   SHIP ('stream_json.py') INPUT(stdin USING PigStreaming(',')) OUTPUT (stdout USING PigStreaming(',')) ;\n\njspy = LOAD 'velib_py/*.txt' USING PigStorage('\\t') AS (arow:chararray);\n\nmatrice = STREAM jspy THROUGH pystream AS \n                (   available_bike_stands:int, \n                    available_bikes:int, \n                    lat:double, \n                    lng:double, \n                    name:chararray, \n                    status:chararray) ;\n\nSTORE matrice INTO 'velib_py_results/firstjob' USING PigStorage('\\t') ;\n")


# On supprime la précédente exécution si besoin puis on vérifie que le répertoire contenant les résultats est vide :

# In[8]:


if client.dfs_exists("velib_py_results/firstjob"):
    client.dfs_rm("velib_py_results/firstjob", recursive=True)
get_ipython().run_line_magic('dfs_mkdir', 'velib_py_results')


# In[9]:


get_ipython().run_line_magic('dfs_ls', 'velib_py_results/')


# In[10]:


get_ipython().run_line_magic('pig_submit', 'json_velib.pig stream_json.py -r redirection')


# In[11]:


get_ipython().run_line_magic('remote_cmd', 'tail redirection.pig.err')


# On récupère les informations qu'on affiche sous forme de dataframe :

# In[12]:


if os.path.exists("velib_exo1.txt") : os.remove("velib_exo1.txt")
client.download_cluster("velib_py_results/firstjob","velib_exo1.txt", merge=True)


# In[13]:


import pandas
df = pandas.read_csv("velib_hd.txt", sep="\t",names=["available_bike_stands","available_bikes","lat","lng","name","status"])
df.head()


# ## Exercice 2 : stations fermées
#     
# Les stations fermées ne le sont pas tout le temps. On veut calculer le ratio minutes/stations fermées / total des minutes/stations. Le script python permettant de lire les données ne change pas, il suffit juste de déclarer les sorties numériques comme telles. Quelques détails sur les tables :
# 
# * ``jspy`` : contient les données brutes dans une liste de fichiers
# * ``matrice`` : d'après le job qui précède, la table contient une ligne par stations et par minute, chaque ligne décrit le status de la station
# * ``grstation`` : table ``matrice`` groupée par status
# * ``fermees`` : pour chaque groupe, on aggrégé le nombre de minutes multipliés par le nombre de vélos
# * ``gr*,dist*`` : distribution du nombre de stations (Y) en fonction du nombre de vélos ou places disponibles

# En cas d'exécution précédentes :

# In[14]:


get_ipython().run_line_magic('dfs_rmr', 'velib_py_results/fermees.txt')
get_ipython().run_line_magic('dfs_rmr', 'velib_py_results/distribution_bikes.txt')
get_ipython().run_line_magic('dfs_rmr', 'velib_py_results/distribution_stands.txt')


# In[15]:


get_ipython().run_cell_magic('PIG', 'json_fermee.pig', "\nDEFINE pystream `python stream_json.py available_bike_stands available_bikes lat lng name status` \n   SHIP ('stream_json.py') INPUT(stdin USING PigStreaming(',')) OUTPUT (stdout USING PigStreaming(',')) ;\n\njspy = LOAD 'velib_py/*.txt' USING PigStorage('\\t') AS (arow:chararray);\n\nmatrice = STREAM jspy THROUGH pystream AS \n                (   available_bike_stands:int, \n                    available_bikes:int, \n                    lat:double, \n                    lng:double, \n                    name:chararray, \n                    status:chararray) ;\n                \ngrstation = GROUP matrice BY status ;  \n\nfermees = FOREACH grstation GENERATE\n                 group\n                ,SUM(matrice.available_bikes) AS available_bikes\n                ,SUM(matrice.available_bike_stands) AS available_bike_stands \n                ;\n\ngr_av   = GROUP matrice BY available_bikes ;\ndist_av = FOREACH gr_av GENERATE group, COUNT(matrice) ;\ngr_pl   = GROUP matrice BY available_bike_stands ;\ndist_pl = FOREACH gr_pl GENERATE group, COUNT(matrice) ;\n\nSTORE fermees INTO 'velib_py_results/fermees.txt' USING PigStorage('\\t') ;\nSTORE dist_av INTO 'velib_py_results/distribution_bikes.txt' USING PigStorage('\\t') ;\nSTORE dist_pl INTO 'velib_py_results/distribution_stands.txt' USING PigStorage('\\t') ;\n")


# In[16]:


get_ipython().run_line_magic('pig_submit', 'json_fermee.pig stream_json.py -r redirection')


# In[17]:


get_ipython().run_line_magic('remote_cmd', 'tail redirection.pig.err')


# In[18]:


get_ipython().run_line_magic('dfs_ls', 'velib_py_results')


# In[19]:


if os.path.exists("distribution_bikes.txt") : os.remove("distribution_bikes.txt")
get_ipython().run_line_magic('remote_down_cluster_merge', 'velib_py_results/distribution_bikes.txt distribution_bikes.txt')


# In[20]:


import pandas
df = pandas.read_csv("distribution_bikes.txt", sep="\t", names=["nb_velos", "nb_stations_minutes"])
df.head()


# In[21]:


df.plot(x="nb_velos",y="nb_stations_minutes",kind="bar",figsize=(16,4))


# In[22]:


if os.path.exists("distribution_stands.txt") : os.remove("distribution_stands.txt")
get_ipython().run_line_magic('remote_down_cluster_merge', 'velib_py_results/distribution_stands.txt distribution_stands.txt')


# In[23]:


df = pandas.read_csv("distribution_stands.txt", sep="\t", names=["nb_places", "nb_stations_minutes"])
df.plot(x="nb_places",y="nb_stations_minutes",kind="bar",figsize=(16,4))


# In[24]:


if os.path.exists("fermees.txt") : os.remove("fermees.txt")
get_ipython().run_line_magic('remote_down_cluster_merge', 'velib_py_results/fermees.txt fermees.txt')


# In[25]:


df = pandas.read_csv("fermees.txt", sep="\t", names=["status", "nb_velos_stations_minutes", "nb_places_stations_minutes"])
df=df.set_index("status")
df = df.T
df["%close"] = df.CLOSED / (df.CLOSED + df.OPEN)
df


# Ce dernier tableau n'est vrai que dans la mesure où les nombres reportées sont fiables lorsque les stations sont fermées.

# ## Exercice 3 : stations fermées, journée complète
#     
# Appliquer cela à une journée complète revient à lancer le même job sur des données plus grandes. On verra bientôt commencer éviter de recopier le job une seconde fois (introduisant une source potentielle d'erreur).

# In[26]:


get_ipython().run_line_magic('dfs_ls', '/user/xavierdupre/')


# In[27]:


get_ipython().run_cell_magic('PIG', 'json_fermee.pig', "\nDEFINE pystream `python stream_json.py available_bike_stands available_bikes lat lng name status` \n   SHIP ('stream_json.py') INPUT(stdin USING PigStreaming(',')) OUTPUT (stdout USING PigStreaming(',')) ;\n\njspy = LOAD '/user/xavierdupre/velib_several_days/*.txt' USING PigStorage('\\t') AS (arow:chararray);\n\nmatrice = STREAM jspy THROUGH pystream AS \n                (   available_bike_stands:int, \n                    available_bikes:int, \n                    lat:double, \n                    lng:double, \n                    name:chararray, \n                    status:chararray) ;\n                \ngrstation = GROUP matrice BY status ;  \n\nfermees = FOREACH grstation GENERATE\n                 group\n                ,SUM(matrice.available_bikes) AS available_bikes\n                ,SUM(matrice.available_bike_stands) AS available_bike_stands \n                ;\n\ngr_av   = GROUP matrice BY available_bikes ;\ndist_av = FOREACH gr_av GENERATE group, COUNT(matrice) ;\ngr_pl   = GROUP matrice BY available_bike_stands ;\ndist_pl = FOREACH gr_pl GENERATE group, COUNT(matrice) ;\n\nSTORE fermees INTO 'velib_py_results_3days/fermees.txt' USING PigStorage('\\t') ;\nSTORE dist_av INTO 'velib_py_results_3days/distribution_bikes.txt' USING PigStorage('\\t') ;\nSTORE dist_pl INTO 'velib_py_results_3days/distribution_stands.txt' USING PigStorage('\\t') ;\n")


# In[28]:


get_ipython().run_line_magic('pigsubmit', 'json_fermee.pig stream_json.py redirection')


# In[29]:


get_ipython().run_line_magic('remote_cmd', 'tail redirection.err')


# In[30]:


get_ipython().run_line_magic('dfs_ls', 'velib_py_results_3days/*')


# On voit que le job a été distribué sur trois machines.

# ## Exercice 4 : astuces
# 
# Les erreurs de PIG ne sont pas très explicite surtout si elles se produisent dans le script python. Un moyen simple de débugger est d'attraper les exceptions produites par python et de les récupérer sous PIG (le reste du job est enlevé). On peut tout-à-fait imaginer ajouter la version de python installée sur le cluster ainsi que la liste de modules

# In[31]:


get_ipython().run_cell_magic('PYTHON', 'stream_json_err.py', '\nimport sys, os\nfor row in sys.stdin:\n    sys.stdout.write(sys.executable + "\\n")\n    sys.stdout.write(str(sys.version) + "\\n")\n    sys.stdout.write(row + "\\n")\n    # pour obtenir les variables d\'environnement\n    #for k in os.environ:\n    #    sys.stdout.write("%s=%s\\n" % (k,os.environ[k]))\n    sys.stdout.flush()\n')


# In[32]:


get_ipython().run_cell_magic('runpy', 'stream_json_err.py', "n'importe quoi\n")


# In[33]:


get_ipython().run_cell_magic('PIG', 'test.pig', "\nDEFINE pystream `python stream_json_err.py` \n   SHIP ('stream_json_err.py') INPUT(stdin USING PigStreaming(',')) OUTPUT (stdout USING PigStreaming(',')) ;\n\njspy = LOAD 'velib_py/*.txt' USING PigStorage('\\t') AS (arow:chararray);\none = LIMIT jspy 1 ;\ninfo = STREAM one THROUGH pystream AS (row:chararray) ;\nSTORE info INTO 'python_info2.txt' ;\n")


# In[34]:


get_ipython().run_line_magic('pig_submit', 'test.pig stream_json_err.py -r redirection')


# In[35]:


get_ipython().run_line_magic('remote_cmd', 'tail redirection.pig.err -n 5')


# In[36]:


if os.path.exists("python_info2.txt") : os.remove("python_info2.txt")
get_ipython().run_line_magic('remote_down_cluster_merge', 'python_info2.txt python_info2.txt')


# In[37]:


get_ipython().run_line_magic('head', 'python_info2.txt')


# In[38]: